# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.batch_summary import BatchSummary
from .query_batches_response_records_item import QueryBatchesResponseRecordsItem


class QueryBatchesResponse(UniversalBaseModel):
    """
    Response body for queries about batches.

    Examples
    --------
    import datetime

    from payabli import BatchSummary, GeneralEvents
    from payabli.query_types import (
        QueryBatchesResponse,
        QueryBatchesResponseRecordsItem,
        QueryBatchesTransfer,
    )

    QueryBatchesResponse(
        summary=BatchSummary(
            page_size=20,
            total_amount=54049.71,
            total_net_amount=0.0,
            total_pages=3,
            total_records=3,
        ),
        records=[
            QueryBatchesResponseRecordsItem(
                id_batch=1049,
                batch_number="batch_2857_combined_08-26-2025_001",
                events_data=[
                    GeneralEvents(
                        description="Created",
                        event_time=datetime.datetime.fromisoformat(
                            "2025-08-25 07:19:27.619000+00:00",
                        ),
                        source="api",
                    )
                ],
                connector_name="GP",
                batch_date=datetime.datetime.fromisoformat(
                    "2025-08-25 20:00:00+00:00",
                ),
                batch_amount=0.0,
                batch_fees_amount=0.0,
                batch_auth_amount=0.0,
                batch_released_amount=0.0,
                batch_hold_amount=0.0,
                batch_returned_amount=0.0,
                batch_refund_amount=0.0,
                batch_split_amount=0.0,
                batch_status=2,
                batch_records=2,
                paypoint_id=187,
                paypoint_name="Gruzya Adventure Outfitters, LLC",
                paypoint_dba="Gruzya Adventure Outfitters",
                parent_org_name="Pilgrim Planner",
                parent_org_id=105,
                entry_name="47f4f8c7e1",
                batch_type=0,
                method="combined",
                expected_deposit_date=datetime.datetime.fromisoformat(
                    "2025-08-26 00:00:00+00:00",
                ),
                transfer_date=datetime.datetime.fromisoformat(
                    "2025-08-26 00:00:00+00:00",
                ),
            ),
            QueryBatchesResponseRecordsItem(
                id_batch=1043,
                batch_number="BT-2023041817-187",
                batch_date=datetime.datetime.fromisoformat(
                    "2023-04-18 17:01:03+00:00",
                ),
                batch_amount=219.02,
                batch_fees_amount=0.0,
                batch_auth_amount=0.0,
                batch_released_amount=0.0,
                batch_hold_amount=0.0,
                batch_returned_amount=0.0,
                batch_refund_amount=0.0,
                batch_split_amount=0.0,
                batch_status=2,
                batch_records=1,
                paypoint_id=187,
                paypoint_name="Gruzya Adventure Outfitters, LLC",
                paypoint_dba="Gruzya Adventure Outfitters",
                parent_org_name="Pilgrim Planner",
                parent_org_id=105,
                entry_name="d193cf9a46",
                batch_type=0,
                method="card",
                expected_deposit_date=datetime.datetime.fromisoformat(
                    "2023-04-19 00:00:00+00:00",
                ),
                transfer_date=datetime.datetime.fromisoformat(
                    "2025-09-02 00:00:00+00:00",
                ),
            ),
            QueryBatchesResponseRecordsItem(
                id_batch=1012,
                batch_number="BT-2023041421-187",
                transfer_identifier="ec310c3d-d4bf-4670-9524-00fcc4ab6a2a",
                events_data=[
                    GeneralEvents(
                        description="Created",
                        event_time=datetime.datetime.fromisoformat(
                            "2023-04-14 21:01:03+00:00",
                        ),
                        source="api",
                    ),
                    GeneralEvents(
                        description="Closed",
                        event_time=datetime.datetime.fromisoformat(
                            "2023-04-15 03:05:10+00:00",
                        ),
                        ref_data="batchId: 1012",
                        source="worker",
                    ),
                ],
                connector_name="GP",
                batch_date=datetime.datetime.fromisoformat(
                    "2023-04-14 21:01:03+00:00",
                ),
                batch_amount=1080.44,
                batch_fees_amount=0.0,
                batch_auth_amount=1080.44,
                batch_released_amount=0.0,
                batch_hold_amount=0.0,
                batch_returned_amount=0.0,
                batch_refund_amount=0.0,
                batch_split_amount=0.0,
                batch_status=2,
                batch_records=4,
                paypoint_id=187,
                paypoint_name="Gruzya Adventure Outfitters, LLC",
                paypoint_dba="Gruzya Adventure Outfitters",
                parent_org_name="Pilgrim Planner",
                parent_org_id=105,
                entry_name="d193cf9a46",
                batch_type=0,
                method="card",
                expected_deposit_date=datetime.datetime.fromisoformat(
                    "2023-04-15 00:00:00+00:00",
                ),
                transfer_date=datetime.datetime.fromisoformat(
                    "2025-09-02 00:00:00+00:00",
                ),
                transfer=QueryBatchesTransfer(
                    transfer_id=5998,
                    transfer_date=datetime.datetime.fromisoformat(
                        "2025-09-02 00:00:00+00:00",
                    ),
                    processor="gp",
                    transfer_status=1,
                    gross_amount=1080.44,
                    charge_back_amount=0.0,
                    returned_amount=0.0,
                    refund_amount=0.0,
                    hold_amount=0.0,
                    released_amount=0.0,
                    billing_fees_amount=0.0,
                    third_party_paid_amount=0.0,
                    adjustments_amount=0.0,
                    net_funded_amount=1080.44,
                ),
            ),
        ],
    )
    """

    records: typing_extensions.Annotated[typing.List[QueryBatchesResponseRecordsItem], FieldMetadata(alias="Records")]
    summary: typing_extensions.Annotated[BatchSummary, FieldMetadata(alias="Summary")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.ach_holder_type import AchHolderType
from ...types.ach_sec_code import AchSecCode
from ...types.batch_number import BatchNumber
from ...types.bill_data import BillData
from ...types.category import Category
from ...types.chargeback_id import ChargebackId
from ...types.created_at import CreatedAt
from ...types.dbaname import Dbaname
from ...types.deposit_date import DepositDate
from ...types.device import Device
from ...types.entrypage_id import EntrypageId
from ...types.entrypointfield import Entrypointfield
from ...types.expected_deposit_date import ExpectedDepositDate
from ...types.external_paypoint_id import ExternalPaypointId
from ...types.fee_amount import FeeAmount
from ...types.legalname import Legalname
from ...types.maskedaccount import Maskedaccount
from ...types.netamountnullable import Netamountnullable
from ...types.operation import Operation
from ...types.order_id import OrderId
from ...types.org_parent_name import OrgParentName
from ...types.orgid import Orgid
from ...types.payor_id import PayorId
from ...types.paypoint_id import PaypointId
from ...types.pending_fee_amount import PendingFeeAmount
from ...types.query_c_fee_transaction import QueryCFeeTransaction
from ...types.query_payment_data import QueryPaymentData
from ...types.query_response_data import QueryResponseData
from ...types.query_transaction_events import QueryTransactionEvents
from ...types.query_transaction_payor_data import QueryTransactionPayorData
from ...types.refund_id import RefundId
from ...types.retrieval_id import RetrievalId
from ...types.returned_id import ReturnedId
from ...types.source import Source
from ...types.split_funding import SplitFunding
from ...types.transaction_time import TransactionTime


class BatchDetailResponseRecord(UniversalBaseModel):
    id: typing_extensions.Annotated[int, FieldMetadata(alias="Id")]
    method: typing_extensions.Annotated[str, FieldMetadata(alias="Method")]
    wallet_type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="WalletType")] = None
    settled_amount: typing_extensions.Annotated[float, FieldMetadata(alias="SettledAmount")]
    type: typing_extensions.Annotated[str, FieldMetadata(alias="Type")]
    batch_number: typing_extensions.Annotated[BatchNumber, FieldMetadata(alias="BatchNumber")]
    batch_amount: typing_extensions.Annotated[float, FieldMetadata(alias="BatchAmount")]
    payment_trans_id: typing_extensions.Annotated[str, FieldMetadata(alias="PaymentTransId")]
    payment_trans_status: typing_extensions.Annotated[int, FieldMetadata(alias="PaymentTransStatus")]
    schedule_reference: typing_extensions.Annotated[int, FieldMetadata(alias="ScheduleReference")]
    gateway_trans_id: typing_extensions.Annotated[str, FieldMetadata(alias="GatewayTransId")]
    order_id: typing_extensions.Annotated[OrderId, FieldMetadata(alias="OrderId")]
    trans_method: typing_extensions.Annotated[str, FieldMetadata(alias="TransMethod")]
    payment_data: typing_extensions.Annotated[typing.Optional[QueryPaymentData], FieldMetadata(alias="PaymentData")] = (
        None
    )
    net_amount: typing_extensions.Annotated[Netamountnullable, FieldMetadata(alias="NetAmount")]
    operation: typing_extensions.Annotated[Operation, FieldMetadata(alias="Operation")]
    category: typing_extensions.Annotated[Category, FieldMetadata(alias="Category")]
    source: typing_extensions.Annotated[typing.Optional[Source], FieldMetadata(alias="Source")] = None
    status: typing_extensions.Annotated[int, FieldMetadata(alias="Status")]
    transaction_time: typing_extensions.Annotated[TransactionTime, FieldMetadata(alias="TransactionTime")]
    customer: typing_extensions.Annotated[
        typing.Optional[QueryTransactionPayorData], FieldMetadata(alias="Customer")
    ] = None
    settlement_date: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="SettlementDate")]
    payment_settlement_status: typing_extensions.Annotated[int, FieldMetadata(alias="PaymentSettlementStatus")]
    batch_status: typing_extensions.Annotated[int, FieldMetadata(alias="BatchStatus")]
    deposit_date: typing_extensions.Annotated[DepositDate, FieldMetadata(alias="DepositDate")]
    expected_deposit_date: typing_extensions.Annotated[ExpectedDepositDate, FieldMetadata(alias="ExpectedDepositDate")]
    masked_account: typing_extensions.Annotated[Maskedaccount, FieldMetadata(alias="MaskedAccount")]
    created_at: typing_extensions.Annotated[CreatedAt, FieldMetadata(alias="CreatedAt")]
    paypoint_legalname: typing_extensions.Annotated[Legalname, FieldMetadata(alias="PaypointLegalname")]
    response_data: typing_extensions.Annotated[
        typing.Optional[QueryResponseData], FieldMetadata(alias="ResponseData")
    ] = None
    paypoint_dbaname: typing_extensions.Annotated[Dbaname, FieldMetadata(alias="PaypointDbaname")]
    parent_org_name: typing_extensions.Annotated[OrgParentName, FieldMetadata(alias="ParentOrgName")]
    parent_org_id: typing_extensions.Annotated[int, FieldMetadata(alias="ParentOrgId")]
    paypoint_entryname: typing_extensions.Annotated[Entrypointfield, FieldMetadata(alias="PaypointEntryname")]
    device_id: typing_extensions.Annotated[typing.Optional[Device], FieldMetadata(alias="DeviceId")] = None
    retrieval_id: typing_extensions.Annotated[RetrievalId, FieldMetadata(alias="RetrievalId")]
    chargeback_id: typing_extensions.Annotated[ChargebackId, FieldMetadata(alias="ChargebackId")]
    ach_holder_type: typing_extensions.Annotated[AchHolderType, FieldMetadata(alias="AchHolderType")]
    ach_sec_code: typing_extensions.Annotated[AchSecCode, FieldMetadata(alias="AchSecCode")]
    connector_name: typing_extensions.Annotated[str, FieldMetadata(alias="ConnectorName")]
    entrypage_id: typing_extensions.Annotated[EntrypageId, FieldMetadata(alias="EntrypageId")]
    fee_amount: typing_extensions.Annotated[FeeAmount, FieldMetadata(alias="FeeAmount")]
    org_id: typing_extensions.Annotated[Orgid, FieldMetadata(alias="OrgId")]
    payor_id: typing_extensions.Annotated[PayorId, FieldMetadata(alias="PayorId")]
    paypoint_id: typing_extensions.Annotated[PaypointId, FieldMetadata(alias="PaypointId")]
    pending_fee_amount: typing_extensions.Annotated[PendingFeeAmount, FieldMetadata(alias="PendingFeeAmount")]
    refund_id: typing_extensions.Annotated[RefundId, FieldMetadata(alias="RefundId")]
    returned_id: typing_extensions.Annotated[ReturnedId, FieldMetadata(alias="ReturnedId")]
    split_funding_instructions: typing_extensions.Annotated[
        SplitFunding, FieldMetadata(alias="splitFundingInstructions")
    ]
    total_amount: typing_extensions.Annotated[float, FieldMetadata(alias="TotalAmount")]
    cfee_transactions: typing_extensions.Annotated[
        typing.List[QueryCFeeTransaction], FieldMetadata(alias="CfeeTransactions")
    ]
    invoice_data: typing_extensions.Annotated[typing.Optional[BillData], FieldMetadata(alias="invoiceData")] = None
    transaction_events: typing_extensions.Annotated[
        typing.List[QueryTransactionEvents], FieldMetadata(alias="TransactionEvents")
    ]
    external_paypoint_id: typing_extensions.Annotated[ExternalPaypointId, FieldMetadata(alias="externalPaypointID")]
    is_hold: typing_extensions.Annotated[int, FieldMetadata(alias="isHold")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .. import core
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.entrypointfield import Entrypointfield
from ..types.payabli_api_response_import import PayabliApiResponseImport
from .raw_client import AsyncRawImportClient, RawImportClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ImportClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawImportClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawImportClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawImportClient
        """
        return self._raw_client

    def import_bills(
        self, entry: str, *, file: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseImport:
        """
        Import a list of bills from a CSV file. See the [Import Guide](/developers/developer-guides/bills-add#import-bills) for more help and an example file.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseImport
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.import_.import_bills(
            entry="8cfec329267",
        )
        """
        _response = self._raw_client.import_bills(entry, file=file, request_options=request_options)
        return _response.data

    def import_customer(
        self,
        entry: Entrypointfield,
        *,
        file: core.File,
        replace_existing: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseImport:
        """
        Import a list of customers from a CSV file. See the [Import Guide](/developers/developer-guides/entities-customers#import-customers) for more help and example files.

        Parameters
        ----------
        entry : Entrypointfield

        file : core.File
            See core.File for more documentation

        replace_existing : typing.Optional[int]
            Flag indicating to replace existing customer with a new record. Possible values: 0 (do not replace), 1 (replace). Default is 0

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseImport
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.import_.import_customer(
            entry="8cfec329267",
        )
        """
        _response = self._raw_client.import_customer(
            entry, file=file, replace_existing=replace_existing, request_options=request_options
        )
        return _response.data

    def import_vendor(
        self, entry: Entrypointfield, *, file: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseImport:
        """
        Import a list of vendors from a CSV file. See the [Import Guide](/developers/developer-guides/entities-vendors#import-vendors) for more help and example files.

        Parameters
        ----------
        entry : Entrypointfield

        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseImport
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.import_.import_vendor(
            entry="8cfec329267",
        )
        """
        _response = self._raw_client.import_vendor(entry, file=file, request_options=request_options)
        return _response.data


class AsyncImportClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawImportClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawImportClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawImportClient
        """
        return self._raw_client

    async def import_bills(
        self, entry: str, *, file: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseImport:
        """
        Import a list of bills from a CSV file. See the [Import Guide](/developers/developer-guides/bills-add#import-bills) for more help and an example file.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseImport
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.import_.import_bills(
                entry="8cfec329267",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.import_bills(entry, file=file, request_options=request_options)
        return _response.data

    async def import_customer(
        self,
        entry: Entrypointfield,
        *,
        file: core.File,
        replace_existing: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseImport:
        """
        Import a list of customers from a CSV file. See the [Import Guide](/developers/developer-guides/entities-customers#import-customers) for more help and example files.

        Parameters
        ----------
        entry : Entrypointfield

        file : core.File
            See core.File for more documentation

        replace_existing : typing.Optional[int]
            Flag indicating to replace existing customer with a new record. Possible values: 0 (do not replace), 1 (replace). Default is 0

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseImport
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.import_.import_customer(
                entry="8cfec329267",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.import_customer(
            entry, file=file, replace_existing=replace_existing, request_options=request_options
        )
        return _response.data

    async def import_vendor(
        self, entry: Entrypointfield, *, file: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseImport:
        """
        Import a list of vendors from a CSV file. See the [Import Guide](/developers/developer-guides/entities-vendors#import-vendors) for more help and example files.

        Parameters
        ----------
        entry : Entrypointfield

        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseImport
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.import_.import_vendor(
                entry="8cfec329267",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.import_vendor(entry, file=file, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.ach_holder import AchHolder
from ...types.ach_sec_code import AchSecCode
from ...types.achaccount import Achaccount
from ...types.achaccounttype import Achaccounttype
from ...types.achrouting import Achrouting


class RequestCreditPaymentMethod(UniversalBaseModel):
    """
    Object describing the ACH payment method to use for transaction.
    """

    ach_account: typing_extensions.Annotated[typing.Optional[Achaccount], FieldMetadata(alias="achAccount")] = None
    ach_account_type: typing_extensions.Annotated[
        typing.Optional[Achaccounttype], FieldMetadata(alias="achAccountType")
    ] = None
    ach_code: typing_extensions.Annotated[typing.Optional[AchSecCode], FieldMetadata(alias="achCode")] = None
    ach_holder: typing_extensions.Annotated[typing.Optional[AchHolder], FieldMetadata(alias="achHolder")] = (
        pydantic.Field(default=None)
    )
    """
    Bank account holder.
    """

    ach_routing: typing_extensions.Annotated[typing.Optional[Achrouting], FieldMetadata(alias="achRouting")] = None
    method: typing.Literal["ach"] = pydantic.Field(default="ach")
    """
    Method to use for the transaction. Must be ACH.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .cardcvv import Cardcvv
from .cardexp import Cardexp
from .cardholder import Cardholder
from .cardnumber import Cardnumber
from .cardzip import Cardzip
from .initiator import Initiator
from .save_if_success import SaveIfSuccess


class PayMethodCredit(UniversalBaseModel):
    cardcvv: typing.Optional[Cardcvv] = None
    cardexp: Cardexp
    card_holder: typing_extensions.Annotated[typing.Optional[Cardholder], FieldMetadata(alias="cardHolder")] = None
    cardnumber: Cardnumber
    cardzip: typing.Optional[Cardzip] = None
    initiator: typing.Optional[Initiator] = None
    method: typing.Literal["card"] = pydantic.Field(default="card")
    """
    Method to use for the transaction. For transactions with a credit or debit card, or a tokenized card, use `card`.
    """

    save_if_success: typing_extensions.Annotated[
        typing.Optional[SaveIfSuccess], FieldMetadata(alias="saveIfSuccess")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PayoutGatewayConnector(UniversalBaseModel):
    configuration: typing.Optional[str] = None
    name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Name")] = None
    mode: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="Mode")] = None
    bank: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Bank")] = None
    descriptor: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Descriptor")] = None
    gateway_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="gatewayID")] = None
    enabled: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="Enabled")] = None
    enable_ach_validation: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="EnableACHValidation")
    ] = None
    test_mode: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="TestMode")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .bin_data import BinData
from .descriptor import Descriptor
from .holdername import Holdername
from .last_modified import LastModified
from .maskedaccount import Maskedaccount


class MethodQueryRecords(UniversalBaseModel):
    bin: typing.Optional[str] = pydantic.Field(default=None)
    """
    The bank identification number (BIN). Null when method is ACH.
    """

    bin_data: typing_extensions.Annotated[typing.Optional[BinData], FieldMetadata(alias="binData")] = None
    descriptor: typing.Optional[Descriptor] = None
    exp_date: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="expDate")] = pydantic.Field(
        default=None
    )
    """
    Expiration date associated to the method (only for card) in format MMYY.
    """

    holder_name: typing_extensions.Annotated[typing.Optional[Holdername], FieldMetadata(alias="holderName")] = None
    id_pmethod: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="idPmethod")] = pydantic.Field(
        default=None
    )
    """
    Method internal ID
    """

    last_updated: typing_extensions.Annotated[typing.Optional[LastModified], FieldMetadata(alias="lastUpdated")] = (
        pydantic.Field(default=None)
    )
    """
    Date of last update
    """

    masked_account: typing_extensions.Annotated[
        typing.Optional[Maskedaccount], FieldMetadata(alias="maskedAccount")
    ] = None
    method: typing.Optional[str] = pydantic.Field(default=None)
    """
    Type of payment vehicle: **ach** or **card**
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.entrypointfield import Entrypointfield
from ...types.payor_data_request import PayorDataRequest
from ...types.source import Source
from ...types.subdomain import Subdomain
from .request_token_storage_payment_method import RequestTokenStoragePaymentMethod


class RequestTokenStorage(UniversalBaseModel):
    customer_data: typing_extensions.Annotated[
        typing.Optional[PayorDataRequest], FieldMetadata(alias="customerData")
    ] = pydantic.Field(default=None)
    """
    Object describing the Customer/Payor owner of payment method. Required for POST requests. Which fields are required depends on the paypoint's custom identifier settings. 
    """

    entry_point: typing_extensions.Annotated[typing.Optional[Entrypointfield], FieldMetadata(alias="entryPoint")] = (
        pydantic.Field(default=None)
    )
    """
    Entrypoint identifier. Required for POST requests.
    """

    fallback_auth: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="fallbackAuth")] = (
        pydantic.Field(default=None)
    )
    """
    When `true`, if tokenization fails, Payabli will attempt an authorization transaction to request a permanent token for the card. If the authorization is successful, the card will be tokenized and the authorization will be voided automatically.
    """

    fallback_auth_amount: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="fallbackAuthAmount")
    ] = pydantic.Field(default=None)
    """
    The amount for the `fallbackAuth` transaction. Defaults to one dollar (`100`).
    """

    method_description: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="methodDescription")] = (
        pydantic.Field(default=None)
    )
    """
    Custom description for stored payment method.
    """

    payment_method: typing_extensions.Annotated[
        typing.Optional[RequestTokenStoragePaymentMethod], FieldMetadata(alias="paymentMethod")
    ] = pydantic.Field(default=None)
    """
    Information about the payment method for the transaction.
    """

    source: typing.Optional[Source] = pydantic.Field(default=None)
    """
    Custom identifier to indicate the source for the request
    """

    subdomain: typing.Optional[Subdomain] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.service_unavailable_error import ServiceUnavailableError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.file import File
from ..types.notification_query_record import NotificationQueryRecord
from ..types.payabli_api_response import PayabliApiResponse
from ..types.payabli_api_response_notifications import PayabliApiResponseNotifications
from .types.add_notification_request import AddNotificationRequest
from .types.update_notification_request import UpdateNotificationRequest

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawNotificationClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def add_notification(
        self, *, request: AddNotificationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponseNotifications]:
        """
        Create a new notification or autogenerated report.

        Parameters
        ----------
        request : AddNotificationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseNotifications]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "Notification",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=AddNotificationRequest, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseNotifications,
                    parse_obj_as(
                        type_=PayabliApiResponseNotifications,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_notification(
        self, n_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponseNotifications]:
        """
        Deletes a single notification or autogenerated report.

        Parameters
        ----------
        n_id : str
            Notification ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseNotifications]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Notification/{jsonable_encoder(n_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseNotifications,
                    parse_obj_as(
                        type_=PayabliApiResponseNotifications,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_notification(
        self, n_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[NotificationQueryRecord]:
        """
        Retrieves a single notification or autogenerated report's details.

        Parameters
        ----------
        n_id : str
            Notification ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[NotificationQueryRecord]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Notification/{jsonable_encoder(n_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    NotificationQueryRecord,
                    parse_obj_as(
                        type_=NotificationQueryRecord,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_notification(
        self, n_id: str, *, request: UpdateNotificationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PayabliApiResponseNotifications]:
        """
        Update a notification or autogenerated report.

        Parameters
        ----------
        n_id : str
            Notification ID.

        request : UpdateNotificationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PayabliApiResponseNotifications]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Notification/{jsonable_encoder(n_id)}",
            method="PUT",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=UpdateNotificationRequest, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseNotifications,
                    parse_obj_as(
                        type_=PayabliApiResponseNotifications,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_report_file(
        self, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[File]:
        """
        Gets a copy of a generated report by ID.

        Parameters
        ----------
        id : int
            Report ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[File]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"Export/notificationReport/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    File,
                    parse_obj_as(
                        type_=File,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawNotificationClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def add_notification(
        self, *, request: AddNotificationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponseNotifications]:
        """
        Create a new notification or autogenerated report.

        Parameters
        ----------
        request : AddNotificationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseNotifications]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "Notification",
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=AddNotificationRequest, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseNotifications,
                    parse_obj_as(
                        type_=PayabliApiResponseNotifications,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_notification(
        self, n_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponseNotifications]:
        """
        Deletes a single notification or autogenerated report.

        Parameters
        ----------
        n_id : str
            Notification ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseNotifications]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Notification/{jsonable_encoder(n_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseNotifications,
                    parse_obj_as(
                        type_=PayabliApiResponseNotifications,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_notification(
        self, n_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[NotificationQueryRecord]:
        """
        Retrieves a single notification or autogenerated report's details.

        Parameters
        ----------
        n_id : str
            Notification ID.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[NotificationQueryRecord]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Notification/{jsonable_encoder(n_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    NotificationQueryRecord,
                    parse_obj_as(
                        type_=NotificationQueryRecord,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_notification(
        self, n_id: str, *, request: UpdateNotificationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PayabliApiResponseNotifications]:
        """
        Update a notification or autogenerated report.

        Parameters
        ----------
        n_id : str
            Notification ID.

        request : UpdateNotificationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PayabliApiResponseNotifications]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Notification/{jsonable_encoder(n_id)}",
            method="PUT",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=UpdateNotificationRequest, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PayabliApiResponseNotifications,
                    parse_obj_as(
                        type_=PayabliApiResponseNotifications,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_report_file(
        self, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[File]:
        """
        Gets a copy of a generated report by ID.

        Parameters
        ----------
        id : int
            Report ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[File]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"Export/notificationReport/{jsonable_encoder(id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    File,
                    parse_obj_as(
                        type_=File,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 503:
                raise ServiceUnavailableError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        PayabliApiResponse,
                        parse_obj_as(
                            type_=PayabliApiResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.accounting_field import AccountingField
from ..types.additional_data_string import AdditionalDataString
from ..types.attachments import Attachments
from ..types.bill_query_response import BillQueryResponse
from ..types.billitems import Billitems
from ..types.billstatus import Billstatus
from ..types.comments import Comments
from ..types.datenullable import Datenullable
from ..types.export_format import ExportFormat
from ..types.file_content import FileContent
from ..types.frequency import Frequency
from ..types.idempotency_key import IdempotencyKey
from ..types.terms import Terms
from ..types.vendor_data import VendorData
from .raw_client import AsyncRawBillClient, RawBillClient
from .types.bill_out_data_scheduled_options import BillOutDataScheduledOptions
from .types.bill_response import BillResponse
from .types.edit_bill_response import EditBillResponse
from .types.get_bill_response import GetBillResponse
from .types.modify_approval_bill_response import ModifyApprovalBillResponse
from .types.set_approved_bill_response import SetApprovedBillResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class BillClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBillClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawBillClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBillClient
        """
        return self._raw_client

    def add_bill(
        self,
        entry: str,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        accounting_field_1: typing.Optional[AccountingField] = OMIT,
        accounting_field_2: typing.Optional[AccountingField] = OMIT,
        additional_data: typing.Optional[AdditionalDataString] = OMIT,
        attachments: typing.Optional[Attachments] = OMIT,
        bill_date: typing.Optional[Datenullable] = OMIT,
        bill_items: typing.Optional[Billitems] = OMIT,
        bill_number: typing.Optional[str] = OMIT,
        comments: typing.Optional[Comments] = OMIT,
        discount: typing.Optional[float] = OMIT,
        due_date: typing.Optional[Datenullable] = OMIT,
        end_date: typing.Optional[Datenullable] = OMIT,
        frequency: typing.Optional[Frequency] = OMIT,
        lot_number: typing.Optional[str] = OMIT,
        mode: typing.Optional[int] = OMIT,
        net_amount: typing.Optional[float] = OMIT,
        scheduled_options: typing.Optional[BillOutDataScheduledOptions] = OMIT,
        status: typing.Optional[Billstatus] = OMIT,
        terms: typing.Optional[Terms] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        vendor: typing.Optional[VendorData] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BillResponse:
        """
        Creates a bill in an entrypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        idempotency_key : typing.Optional[IdempotencyKey]

        accounting_field_1 : typing.Optional[AccountingField]

        accounting_field_2 : typing.Optional[AccountingField]

        additional_data : typing.Optional[AdditionalDataString]

        attachments : typing.Optional[Attachments]
            An array of bill images. Attachments aren't required, but we strongly recommend including them. Including a bill image can make payouts smoother and prevent delays. You can include either the Base64-encoded file content, or you can include an fURL to a public file. The maximum file size for image uploads is 30 MB.

        bill_date : typing.Optional[Datenullable]
            Date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        bill_items : typing.Optional[Billitems]

        bill_number : typing.Optional[str]
            Unique identifier for the bill. Required when adding a bill.

        comments : typing.Optional[Comments]

        discount : typing.Optional[float]
            Discount amount applied to the bill.

        due_date : typing.Optional[Datenullable]
            Due date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        end_date : typing.Optional[Datenullable]
            End Date for scheduled bills. Applied only in `Mode` = 1. Accepted formats: YYYY-MM-DD, MM/DD/YYYY

        frequency : typing.Optional[Frequency]
            Frequency for scheduled bills. Applied only in `Mode` = 1.

        lot_number : typing.Optional[str]
            Lot number associated with the bill.

        mode : typing.Optional[int]
            Bill mode: value `0` for one-time bills, `1` for scheduled bills.

        net_amount : typing.Optional[float]
            Net Amount owed in bill. Required when adding a bill.

        scheduled_options : typing.Optional[BillOutDataScheduledOptions]
            Options for scheduled bills.

        status : typing.Optional[Billstatus]

        terms : typing.Optional[Terms]

        total_amount : typing.Optional[float]
            Total amount of the bill.

        vendor : typing.Optional[VendorData]
            The vendor associated with the bill. Although you can create a vendor in a create bill request, Payabli recommends creating a vendor separately and passing a valid `vendorNumber` here. At minimum, the `vendorNumber` is required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillResponse
            Success

        Examples
        --------
        import datetime

        from payabli import BillItem, FileContent, VendorData, payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.bill.add_bill(
            entry="8cfec329267",
            bill_number="ABC-123",
            net_amount=3762.87,
            bill_date=datetime.date.fromisoformat(
                "2024-07-01",
            ),
            due_date=datetime.date.fromisoformat(
                "2024-07-01",
            ),
            comments="Deposit for materials",
            bill_items=[
                BillItem(
                    item_product_code="M-DEPOSIT",
                    item_product_name="Materials deposit",
                    item_description="Deposit for materials",
                    item_commodity_code="010",
                    item_unit_of_measure="SqFt",
                    item_cost=5.0,
                    item_qty=1,
                    item_mode=0,
                    item_categories=["deposits"],
                    item_total_amount=123.0,
                    item_tax_amount=7.0,
                    item_tax_rate=0.075,
                )
            ],
            mode=0,
            accounting_field_1="MyInternalId",
            vendor=VendorData(
                vendor_number="1234-A",
            ),
            end_date=datetime.date.fromisoformat(
                "2024-07-01",
            ),
            frequency="monthly",
            terms="NET30",
            status=-99,
            attachments=[
                FileContent(
                    ftype="pdf",
                    filename="my-doc.pdf",
                    furl="https://mysite.com/my-doc.pdf",
                )
            ],
        )
        """
        _response = self._raw_client.add_bill(
            entry,
            idempotency_key=idempotency_key,
            accounting_field_1=accounting_field_1,
            accounting_field_2=accounting_field_2,
            additional_data=additional_data,
            attachments=attachments,
            bill_date=bill_date,
            bill_items=bill_items,
            bill_number=bill_number,
            comments=comments,
            discount=discount,
            due_date=due_date,
            end_date=end_date,
            frequency=frequency,
            lot_number=lot_number,
            mode=mode,
            net_amount=net_amount,
            scheduled_options=scheduled_options,
            status=status,
            terms=terms,
            total_amount=total_amount,
            vendor=vendor,
            request_options=request_options,
        )
        return _response.data

    def delete_attached_from_bill(
        self,
        filename: str,
        id_bill: int,
        *,
        return_object: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BillResponse:
        """
        Delete a file attached to a bill.

        Parameters
        ----------
        filename : str
            The filename in Payabli. Filename is `zipName` in response to a
            request to `/api/Invoice/{idInvoice}`. Here, the filename is
            `0_Bill.pdf`.

            ```json
              "DocumentsRef": {
                "zipfile": "inva_269.zip",
                "filelist": [
                  {
                    "originalName": "Bill.pdf",
                    "zipName": "0_Bill.pdf",
                    "descriptor": null
                  }
                ]
              }
              ```

        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        return_object : typing.Optional[bool]
            When `true`, the request returns the file content as a Base64-encoded string.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.bill.delete_attached_from_bill(
            filename="0_Bill.pdf",
            id_bill=285,
        )
        """
        _response = self._raw_client.delete_attached_from_bill(
            filename, id_bill, return_object=return_object, request_options=request_options
        )
        return _response.data

    def delete_bill(self, id_bill: int, *, request_options: typing.Optional[RequestOptions] = None) -> BillResponse:
        """
        Deletes a bill by ID.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.bill.delete_bill(
            id_bill=285,
        )
        """
        _response = self._raw_client.delete_bill(id_bill, request_options=request_options)
        return _response.data

    def edit_bill(
        self,
        id_bill: int,
        *,
        accounting_field_1: typing.Optional[AccountingField] = OMIT,
        accounting_field_2: typing.Optional[AccountingField] = OMIT,
        additional_data: typing.Optional[AdditionalDataString] = OMIT,
        attachments: typing.Optional[Attachments] = OMIT,
        bill_date: typing.Optional[Datenullable] = OMIT,
        bill_items: typing.Optional[Billitems] = OMIT,
        bill_number: typing.Optional[str] = OMIT,
        comments: typing.Optional[Comments] = OMIT,
        discount: typing.Optional[float] = OMIT,
        due_date: typing.Optional[Datenullable] = OMIT,
        end_date: typing.Optional[Datenullable] = OMIT,
        frequency: typing.Optional[Frequency] = OMIT,
        lot_number: typing.Optional[str] = OMIT,
        mode: typing.Optional[int] = OMIT,
        net_amount: typing.Optional[float] = OMIT,
        scheduled_options: typing.Optional[BillOutDataScheduledOptions] = OMIT,
        status: typing.Optional[Billstatus] = OMIT,
        terms: typing.Optional[Terms] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        vendor: typing.Optional[VendorData] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EditBillResponse:
        """
        Updates a bill by ID.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        accounting_field_1 : typing.Optional[AccountingField]

        accounting_field_2 : typing.Optional[AccountingField]

        additional_data : typing.Optional[AdditionalDataString]

        attachments : typing.Optional[Attachments]
            An array of bill images. Attachments aren't required, but we strongly recommend including them. Including a bill image can make payouts smoother and prevent delays. You can include either the Base64-encoded file content, or you can include an fURL to a public file. The maximum file size for image uploads is 30 MB.

        bill_date : typing.Optional[Datenullable]
            Date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        bill_items : typing.Optional[Billitems]

        bill_number : typing.Optional[str]
            Unique identifier for the bill. Required when adding a bill.

        comments : typing.Optional[Comments]

        discount : typing.Optional[float]
            Discount amount applied to the bill.

        due_date : typing.Optional[Datenullable]
            Due date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        end_date : typing.Optional[Datenullable]
            End Date for scheduled bills. Applied only in `Mode` = 1. Accepted formats: YYYY-MM-DD, MM/DD/YYYY

        frequency : typing.Optional[Frequency]
            Frequency for scheduled bills. Applied only in `Mode` = 1.

        lot_number : typing.Optional[str]
            Lot number associated with the bill.

        mode : typing.Optional[int]
            Bill mode: value `0` for one-time bills, `1` for scheduled bills.

        net_amount : typing.Optional[float]
            Net Amount owed in bill. Required when adding a bill.

        scheduled_options : typing.Optional[BillOutDataScheduledOptions]
            Options for scheduled bills.

        status : typing.Optional[Billstatus]

        terms : typing.Optional[Terms]

        total_amount : typing.Optional[float]
            Total amount of the bill.

        vendor : typing.Optional[VendorData]
            The vendor associated with the bill. Although you can create a vendor in a create bill request, Payabli recommends creating a vendor separately and passing a valid `vendorNumber` here. At minimum, the `vendorNumber` is required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditBillResponse
            Success

        Examples
        --------
        import datetime

        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.bill.edit_bill(
            id_bill=285,
            net_amount=3762.87,
            bill_date=datetime.date.fromisoformat(
                "2025-07-01",
            ),
        )
        """
        _response = self._raw_client.edit_bill(
            id_bill,
            accounting_field_1=accounting_field_1,
            accounting_field_2=accounting_field_2,
            additional_data=additional_data,
            attachments=attachments,
            bill_date=bill_date,
            bill_items=bill_items,
            bill_number=bill_number,
            comments=comments,
            discount=discount,
            due_date=due_date,
            end_date=end_date,
            frequency=frequency,
            lot_number=lot_number,
            mode=mode,
            net_amount=net_amount,
            scheduled_options=scheduled_options,
            status=status,
            terms=terms,
            total_amount=total_amount,
            vendor=vendor,
            request_options=request_options,
        )
        return _response.data

    def get_attached_from_bill(
        self,
        filename: str,
        id_bill: int,
        *,
        return_object: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FileContent:
        """
        Retrieves a file attached to a bill, either as a binary file or as a Base64-encoded string.

        Parameters
        ----------
        filename : str
            The filename in Payabli. Filename is `zipName` in response to a request to `/api/Invoice/{idInvoice}`. Here, the filename is `0_Bill.pdf``.
            "DocumentsRef": {
              "zipfile": "inva_269.zip",
              "filelist": [
                {
                  "originalName": "Bill.pdf",
                  "zipName": "0_Bill.pdf",
                  "descriptor": null
                }
              ]
            }

        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        return_object : typing.Optional[bool]
            When `true`, the request returns the file content as a Base64-encoded string.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FileContent
            A successful response returns a binary file when `returnObject` is `false`. When `returnObject` is `true`, the response contains the file content as a Base64-encoded string in an object. Due to technical limitations, only the object response is documented here.

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.bill.get_attached_from_bill(
            filename="0_Bill.pdf",
            id_bill=285,
            return_object=True,
        )
        """
        _response = self._raw_client.get_attached_from_bill(
            filename, id_bill, return_object=return_object, request_options=request_options
        )
        return _response.data

    def get_bill(self, id_bill: int, *, request_options: typing.Optional[RequestOptions] = None) -> GetBillResponse:
        """
        Retrieves a bill by ID from an entrypoint.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBillResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.bill.get_bill(
            id_bill=285,
        )
        """
        _response = self._raw_client.get_bill(id_bill, request_options=request_options)
        return _response.data

    def list_bills(
        self,
        entry: str,
        *,
        export_format: typing.Optional[ExportFormat] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BillQueryResponse:
        """
        Retrieve a list of bills for an entrypoint. Use filters to limit results. Include the `exportFormat` query parameter to return the results as a file instead of a JSON response.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        export_format : typing.Optional[ExportFormat]

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query
            <Info>
              **You must remove `parameters=` from the request before you send it, otherwise Payabli will ignore the filters.**

              Because of a technical limitation, you can't make a request that includes filters from the API console on this page. The response isn't filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

              For example:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?parameters=totalAmount(gt)=1000&limitRecord=20

              should become:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?totalAmount(gt)=1000&limitRecord=20
            </Info>
            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `frequency` (`in`, `nin`, `ne`, `eq`)
            - `method` (`in`, `nin`, `eq`, `ne`)
            - `event` (`in`, `nin`, `eq`, `ne`)
            - `target` (`ct`, `nct`, `eq`, `ne`)
            - `status` (`eq`, `ne`)
            - `approvalUserId` (`eq`, `ne`)
            - `parentOrgId` (`ne`, `eq`, `nin`, `in`)
            - `approvalUserEmail` (`eq`, `ne`)
            - `scheduleId` (`ne`, `eq`)

            List of comparison accepted - enclosed between parentheses:
            - `eq` or empty => equal
            - `gt` => greater than
            - `ge` => greater or equal
            - `lt` => less than
            - `le` => less or equal
            - `ne` => not equal
            - `ct` => contains
            - `nct` => not contains
            - `in` => inside array
            - `nin` => not inside array

            List of parameters accepted:
            - `limitRecord` : max number of records for query (default="20", "0" or negative value for all)
            - `fromRecord` : initial record in query
            Example: `totalAmount(gt)=20` returns all records with a `totalAmount` that's greater than 20.00

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillQueryResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.bill.list_bills(
            entry="8cfec329267",
            from_record=251,
            limit_record=0,
            sort_by="desc(field_name)",
        )
        """
        _response = self._raw_client.list_bills(
            entry,
            export_format=export_format,
            from_record=from_record,
            limit_record=limit_record,
            parameters=parameters,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    def list_bills_org(
        self,
        org_id: int,
        *,
        export_format: typing.Optional[ExportFormat] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BillQueryResponse:
        """
        Retrieve a list of bills for an organization. Use filters to limit results. Include the `exportFormat` query parameter to return the results as a file instead of a JSON response.

        Parameters
        ----------
        org_id : int
            The numeric identifier for organization, assigned by Payabli.

        export_format : typing.Optional[ExportFormat]

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query
            <Info>
              **You must remove `parameters=` from the request before you send it, otherwise Payabli will ignore the filters.**

              Because of a technical limitation, you can't make a request that includes filters from the API console on this page. The response isn't filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

              For example:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?parameters=totalAmount(gt)=1000&limitRecord=20

              should become:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?totalAmount(gt)=1000&limitRecord=20
            </Info>
            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `frequency` (in, nin, ne, eq)
            - `method` (in, nin, eq, ne)
            - `event` (in, nin, eq, ne)
            - `target` (ct, nct, eq, ne)
            - `status` (eq, ne)
            - `parentOrgId` (ne, eq, nin, in)
            - `approvalUserId` (eq, ne)
            - `approvalUserEmail` (eq, ne)

            List of comparison accepted - enclosed between parentheses:
            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: totalAmount(gt)=20 return all records with totalAmount greater than 20.00

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillQueryResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.bill.list_bills_org(
            org_id=123,
            from_record=251,
            limit_record=0,
            sort_by="desc(field_name)",
        )
        """
        _response = self._raw_client.list_bills_org(
            org_id,
            export_format=export_format,
            from_record=from_record,
            limit_record=limit_record,
            parameters=parameters,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    def modify_approval_bill(
        self, id_bill: int, *, request: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> ModifyApprovalBillResponse:
        """
        Modify the list of users the bill is sent to for approval.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request : typing.Sequence[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModifyApprovalBillResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.bill.modify_approval_bill(
            id_bill=285,
            request=["string"],
        )
        """
        _response = self._raw_client.modify_approval_bill(id_bill, request=request, request_options=request_options)
        return _response.data

    def send_to_approval_bill(
        self,
        id_bill: int,
        *,
        request: typing.Sequence[str],
        autocreate_user: typing.Optional[bool] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BillResponse:
        """
        Send a bill to a user or list of users to approve.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request : typing.Sequence[str]

        autocreate_user : typing.Optional[bool]
            Automatically create the target user for approval if they don't exist.

        idempotency_key : typing.Optional[IdempotencyKey]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.bill.send_to_approval_bill(
            id_bill=285,
            idempotency_key="6B29FC40-CA47-1067-B31D-00DD010662DA",
            request=["string"],
        )
        """
        _response = self._raw_client.send_to_approval_bill(
            id_bill,
            request=request,
            autocreate_user=autocreate_user,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return _response.data

    def set_approved_bill(
        self,
        approved: str,
        id_bill: int,
        *,
        email: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SetApprovedBillResponse:
        """
        Approve or disapprove a bill by ID.

        Parameters
        ----------
        approved : str
            String representing the approved status. Accepted values: 'true' or 'false'.

        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        email : typing.Optional[str]
            Email or username of user modifying approval status.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetApprovedBillResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.bill.set_approved_bill(
            approved="true",
            id_bill=285,
        )
        """
        _response = self._raw_client.set_approved_bill(approved, id_bill, email=email, request_options=request_options)
        return _response.data


class AsyncBillClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBillClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawBillClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBillClient
        """
        return self._raw_client

    async def add_bill(
        self,
        entry: str,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        accounting_field_1: typing.Optional[AccountingField] = OMIT,
        accounting_field_2: typing.Optional[AccountingField] = OMIT,
        additional_data: typing.Optional[AdditionalDataString] = OMIT,
        attachments: typing.Optional[Attachments] = OMIT,
        bill_date: typing.Optional[Datenullable] = OMIT,
        bill_items: typing.Optional[Billitems] = OMIT,
        bill_number: typing.Optional[str] = OMIT,
        comments: typing.Optional[Comments] = OMIT,
        discount: typing.Optional[float] = OMIT,
        due_date: typing.Optional[Datenullable] = OMIT,
        end_date: typing.Optional[Datenullable] = OMIT,
        frequency: typing.Optional[Frequency] = OMIT,
        lot_number: typing.Optional[str] = OMIT,
        mode: typing.Optional[int] = OMIT,
        net_amount: typing.Optional[float] = OMIT,
        scheduled_options: typing.Optional[BillOutDataScheduledOptions] = OMIT,
        status: typing.Optional[Billstatus] = OMIT,
        terms: typing.Optional[Terms] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        vendor: typing.Optional[VendorData] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BillResponse:
        """
        Creates a bill in an entrypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        idempotency_key : typing.Optional[IdempotencyKey]

        accounting_field_1 : typing.Optional[AccountingField]

        accounting_field_2 : typing.Optional[AccountingField]

        additional_data : typing.Optional[AdditionalDataString]

        attachments : typing.Optional[Attachments]
            An array of bill images. Attachments aren't required, but we strongly recommend including them. Including a bill image can make payouts smoother and prevent delays. You can include either the Base64-encoded file content, or you can include an fURL to a public file. The maximum file size for image uploads is 30 MB.

        bill_date : typing.Optional[Datenullable]
            Date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        bill_items : typing.Optional[Billitems]

        bill_number : typing.Optional[str]
            Unique identifier for the bill. Required when adding a bill.

        comments : typing.Optional[Comments]

        discount : typing.Optional[float]
            Discount amount applied to the bill.

        due_date : typing.Optional[Datenullable]
            Due date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        end_date : typing.Optional[Datenullable]
            End Date for scheduled bills. Applied only in `Mode` = 1. Accepted formats: YYYY-MM-DD, MM/DD/YYYY

        frequency : typing.Optional[Frequency]
            Frequency for scheduled bills. Applied only in `Mode` = 1.

        lot_number : typing.Optional[str]
            Lot number associated with the bill.

        mode : typing.Optional[int]
            Bill mode: value `0` for one-time bills, `1` for scheduled bills.

        net_amount : typing.Optional[float]
            Net Amount owed in bill. Required when adding a bill.

        scheduled_options : typing.Optional[BillOutDataScheduledOptions]
            Options for scheduled bills.

        status : typing.Optional[Billstatus]

        terms : typing.Optional[Terms]

        total_amount : typing.Optional[float]
            Total amount of the bill.

        vendor : typing.Optional[VendorData]
            The vendor associated with the bill. Although you can create a vendor in a create bill request, Payabli recommends creating a vendor separately and passing a valid `vendorNumber` here. At minimum, the `vendorNumber` is required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillResponse
            Success

        Examples
        --------
        import asyncio
        import datetime

        from payabli import Asyncpayabli, BillItem, FileContent, VendorData

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.bill.add_bill(
                entry="8cfec329267",
                bill_number="ABC-123",
                net_amount=3762.87,
                bill_date=datetime.date.fromisoformat(
                    "2024-07-01",
                ),
                due_date=datetime.date.fromisoformat(
                    "2024-07-01",
                ),
                comments="Deposit for materials",
                bill_items=[
                    BillItem(
                        item_product_code="M-DEPOSIT",
                        item_product_name="Materials deposit",
                        item_description="Deposit for materials",
                        item_commodity_code="010",
                        item_unit_of_measure="SqFt",
                        item_cost=5.0,
                        item_qty=1,
                        item_mode=0,
                        item_categories=["deposits"],
                        item_total_amount=123.0,
                        item_tax_amount=7.0,
                        item_tax_rate=0.075,
                    )
                ],
                mode=0,
                accounting_field_1="MyInternalId",
                vendor=VendorData(
                    vendor_number="1234-A",
                ),
                end_date=datetime.date.fromisoformat(
                    "2024-07-01",
                ),
                frequency="monthly",
                terms="NET30",
                status=-99,
                attachments=[
                    FileContent(
                        ftype="pdf",
                        filename="my-doc.pdf",
                        furl="https://mysite.com/my-doc.pdf",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add_bill(
            entry,
            idempotency_key=idempotency_key,
            accounting_field_1=accounting_field_1,
            accounting_field_2=accounting_field_2,
            additional_data=additional_data,
            attachments=attachments,
            bill_date=bill_date,
            bill_items=bill_items,
            bill_number=bill_number,
            comments=comments,
            discount=discount,
            due_date=due_date,
            end_date=end_date,
            frequency=frequency,
            lot_number=lot_number,
            mode=mode,
            net_amount=net_amount,
            scheduled_options=scheduled_options,
            status=status,
            terms=terms,
            total_amount=total_amount,
            vendor=vendor,
            request_options=request_options,
        )
        return _response.data

    async def delete_attached_from_bill(
        self,
        filename: str,
        id_bill: int,
        *,
        return_object: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BillResponse:
        """
        Delete a file attached to a bill.

        Parameters
        ----------
        filename : str
            The filename in Payabli. Filename is `zipName` in response to a
            request to `/api/Invoice/{idInvoice}`. Here, the filename is
            `0_Bill.pdf`.

            ```json
              "DocumentsRef": {
                "zipfile": "inva_269.zip",
                "filelist": [
                  {
                    "originalName": "Bill.pdf",
                    "zipName": "0_Bill.pdf",
                    "descriptor": null
                  }
                ]
              }
              ```

        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        return_object : typing.Optional[bool]
            When `true`, the request returns the file content as a Base64-encoded string.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.bill.delete_attached_from_bill(
                filename="0_Bill.pdf",
                id_bill=285,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_attached_from_bill(
            filename, id_bill, return_object=return_object, request_options=request_options
        )
        return _response.data

    async def delete_bill(
        self, id_bill: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BillResponse:
        """
        Deletes a bill by ID.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.bill.delete_bill(
                id_bill=285,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_bill(id_bill, request_options=request_options)
        return _response.data

    async def edit_bill(
        self,
        id_bill: int,
        *,
        accounting_field_1: typing.Optional[AccountingField] = OMIT,
        accounting_field_2: typing.Optional[AccountingField] = OMIT,
        additional_data: typing.Optional[AdditionalDataString] = OMIT,
        attachments: typing.Optional[Attachments] = OMIT,
        bill_date: typing.Optional[Datenullable] = OMIT,
        bill_items: typing.Optional[Billitems] = OMIT,
        bill_number: typing.Optional[str] = OMIT,
        comments: typing.Optional[Comments] = OMIT,
        discount: typing.Optional[float] = OMIT,
        due_date: typing.Optional[Datenullable] = OMIT,
        end_date: typing.Optional[Datenullable] = OMIT,
        frequency: typing.Optional[Frequency] = OMIT,
        lot_number: typing.Optional[str] = OMIT,
        mode: typing.Optional[int] = OMIT,
        net_amount: typing.Optional[float] = OMIT,
        scheduled_options: typing.Optional[BillOutDataScheduledOptions] = OMIT,
        status: typing.Optional[Billstatus] = OMIT,
        terms: typing.Optional[Terms] = OMIT,
        total_amount: typing.Optional[float] = OMIT,
        vendor: typing.Optional[VendorData] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EditBillResponse:
        """
        Updates a bill by ID.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        accounting_field_1 : typing.Optional[AccountingField]

        accounting_field_2 : typing.Optional[AccountingField]

        additional_data : typing.Optional[AdditionalDataString]

        attachments : typing.Optional[Attachments]
            An array of bill images. Attachments aren't required, but we strongly recommend including them. Including a bill image can make payouts smoother and prevent delays. You can include either the Base64-encoded file content, or you can include an fURL to a public file. The maximum file size for image uploads is 30 MB.

        bill_date : typing.Optional[Datenullable]
            Date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        bill_items : typing.Optional[Billitems]

        bill_number : typing.Optional[str]
            Unique identifier for the bill. Required when adding a bill.

        comments : typing.Optional[Comments]

        discount : typing.Optional[float]
            Discount amount applied to the bill.

        due_date : typing.Optional[Datenullable]
            Due date of bill. Accepted formats: YYYY-MM-DD, MM/DD/YYYY.

        end_date : typing.Optional[Datenullable]
            End Date for scheduled bills. Applied only in `Mode` = 1. Accepted formats: YYYY-MM-DD, MM/DD/YYYY

        frequency : typing.Optional[Frequency]
            Frequency for scheduled bills. Applied only in `Mode` = 1.

        lot_number : typing.Optional[str]
            Lot number associated with the bill.

        mode : typing.Optional[int]
            Bill mode: value `0` for one-time bills, `1` for scheduled bills.

        net_amount : typing.Optional[float]
            Net Amount owed in bill. Required when adding a bill.

        scheduled_options : typing.Optional[BillOutDataScheduledOptions]
            Options for scheduled bills.

        status : typing.Optional[Billstatus]

        terms : typing.Optional[Terms]

        total_amount : typing.Optional[float]
            Total amount of the bill.

        vendor : typing.Optional[VendorData]
            The vendor associated with the bill. Although you can create a vendor in a create bill request, Payabli recommends creating a vendor separately and passing a valid `vendorNumber` here. At minimum, the `vendorNumber` is required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EditBillResponse
            Success

        Examples
        --------
        import asyncio
        import datetime

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.bill.edit_bill(
                id_bill=285,
                net_amount=3762.87,
                bill_date=datetime.date.fromisoformat(
                    "2025-07-01",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.edit_bill(
            id_bill,
            accounting_field_1=accounting_field_1,
            accounting_field_2=accounting_field_2,
            additional_data=additional_data,
            attachments=attachments,
            bill_date=bill_date,
            bill_items=bill_items,
            bill_number=bill_number,
            comments=comments,
            discount=discount,
            due_date=due_date,
            end_date=end_date,
            frequency=frequency,
            lot_number=lot_number,
            mode=mode,
            net_amount=net_amount,
            scheduled_options=scheduled_options,
            status=status,
            terms=terms,
            total_amount=total_amount,
            vendor=vendor,
            request_options=request_options,
        )
        return _response.data

    async def get_attached_from_bill(
        self,
        filename: str,
        id_bill: int,
        *,
        return_object: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FileContent:
        """
        Retrieves a file attached to a bill, either as a binary file or as a Base64-encoded string.

        Parameters
        ----------
        filename : str
            The filename in Payabli. Filename is `zipName` in response to a request to `/api/Invoice/{idInvoice}`. Here, the filename is `0_Bill.pdf``.
            "DocumentsRef": {
              "zipfile": "inva_269.zip",
              "filelist": [
                {
                  "originalName": "Bill.pdf",
                  "zipName": "0_Bill.pdf",
                  "descriptor": null
                }
              ]
            }

        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        return_object : typing.Optional[bool]
            When `true`, the request returns the file content as a Base64-encoded string.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FileContent
            A successful response returns a binary file when `returnObject` is `false`. When `returnObject` is `true`, the response contains the file content as a Base64-encoded string in an object. Due to technical limitations, only the object response is documented here.

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.bill.get_attached_from_bill(
                filename="0_Bill.pdf",
                id_bill=285,
                return_object=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_attached_from_bill(
            filename, id_bill, return_object=return_object, request_options=request_options
        )
        return _response.data

    async def get_bill(
        self, id_bill: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetBillResponse:
        """
        Retrieves a bill by ID from an entrypoint.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetBillResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.bill.get_bill(
                id_bill=285,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_bill(id_bill, request_options=request_options)
        return _response.data

    async def list_bills(
        self,
        entry: str,
        *,
        export_format: typing.Optional[ExportFormat] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BillQueryResponse:
        """
        Retrieve a list of bills for an entrypoint. Use filters to limit results. Include the `exportFormat` query parameter to return the results as a file instead of a JSON response.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        export_format : typing.Optional[ExportFormat]

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query
            <Info>
              **You must remove `parameters=` from the request before you send it, otherwise Payabli will ignore the filters.**

              Because of a technical limitation, you can't make a request that includes filters from the API console on this page. The response isn't filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

              For example:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?parameters=totalAmount(gt)=1000&limitRecord=20

              should become:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?totalAmount(gt)=1000&limitRecord=20
            </Info>
            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `frequency` (`in`, `nin`, `ne`, `eq`)
            - `method` (`in`, `nin`, `eq`, `ne`)
            - `event` (`in`, `nin`, `eq`, `ne`)
            - `target` (`ct`, `nct`, `eq`, `ne`)
            - `status` (`eq`, `ne`)
            - `approvalUserId` (`eq`, `ne`)
            - `parentOrgId` (`ne`, `eq`, `nin`, `in`)
            - `approvalUserEmail` (`eq`, `ne`)
            - `scheduleId` (`ne`, `eq`)

            List of comparison accepted - enclosed between parentheses:
            - `eq` or empty => equal
            - `gt` => greater than
            - `ge` => greater or equal
            - `lt` => less than
            - `le` => less or equal
            - `ne` => not equal
            - `ct` => contains
            - `nct` => not contains
            - `in` => inside array
            - `nin` => not inside array

            List of parameters accepted:
            - `limitRecord` : max number of records for query (default="20", "0" or negative value for all)
            - `fromRecord` : initial record in query
            Example: `totalAmount(gt)=20` returns all records with a `totalAmount` that's greater than 20.00

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillQueryResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.bill.list_bills(
                entry="8cfec329267",
                from_record=251,
                limit_record=0,
                sort_by="desc(field_name)",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_bills(
            entry,
            export_format=export_format,
            from_record=from_record,
            limit_record=limit_record,
            parameters=parameters,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    async def list_bills_org(
        self,
        org_id: int,
        *,
        export_format: typing.Optional[ExportFormat] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BillQueryResponse:
        """
        Retrieve a list of bills for an organization. Use filters to limit results. Include the `exportFormat` query parameter to return the results as a file instead of a JSON response.

        Parameters
        ----------
        org_id : int
            The numeric identifier for organization, assigned by Payabli.

        export_format : typing.Optional[ExportFormat]

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query
            <Info>
              **You must remove `parameters=` from the request before you send it, otherwise Payabli will ignore the filters.**

              Because of a technical limitation, you can't make a request that includes filters from the API console on this page. The response isn't filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

              For example:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?parameters=totalAmount(gt)=1000&limitRecord=20

              should become:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?totalAmount(gt)=1000&limitRecord=20
            </Info>
            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `frequency` (in, nin, ne, eq)
            - `method` (in, nin, eq, ne)
            - `event` (in, nin, eq, ne)
            - `target` (ct, nct, eq, ne)
            - `status` (eq, ne)
            - `parentOrgId` (ne, eq, nin, in)
            - `approvalUserId` (eq, ne)
            - `approvalUserEmail` (eq, ne)

            List of comparison accepted - enclosed between parentheses:
            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: totalAmount(gt)=20 return all records with totalAmount greater than 20.00

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillQueryResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.bill.list_bills_org(
                org_id=123,
                from_record=251,
                limit_record=0,
                sort_by="desc(field_name)",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_bills_org(
            org_id,
            export_format=export_format,
            from_record=from_record,
            limit_record=limit_record,
            parameters=parameters,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    async def modify_approval_bill(
        self, id_bill: int, *, request: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> ModifyApprovalBillResponse:
        """
        Modify the list of users the bill is sent to for approval.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request : typing.Sequence[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ModifyApprovalBillResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.bill.modify_approval_bill(
                id_bill=285,
                request=["string"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.modify_approval_bill(
            id_bill, request=request, request_options=request_options
        )
        return _response.data

    async def send_to_approval_bill(
        self,
        id_bill: int,
        *,
        request: typing.Sequence[str],
        autocreate_user: typing.Optional[bool] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BillResponse:
        """
        Send a bill to a user or list of users to approve.

        Parameters
        ----------
        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        request : typing.Sequence[str]

        autocreate_user : typing.Optional[bool]
            Automatically create the target user for approval if they don't exist.

        idempotency_key : typing.Optional[IdempotencyKey]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.bill.send_to_approval_bill(
                id_bill=285,
                idempotency_key="6B29FC40-CA47-1067-B31D-00DD010662DA",
                request=["string"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.send_to_approval_bill(
            id_bill,
            request=request,
            autocreate_user=autocreate_user,
            idempotency_key=idempotency_key,
            request_options=request_options,
        )
        return _response.data

    async def set_approved_bill(
        self,
        approved: str,
        id_bill: int,
        *,
        email: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SetApprovedBillResponse:
        """
        Approve or disapprove a bill by ID.

        Parameters
        ----------
        approved : str
            String representing the approved status. Accepted values: 'true' or 'false'.

        id_bill : int
            Payabli ID for the bill. Get this ID by querying `/api/Query/bills/` for the entrypoint or the organization.

        email : typing.Optional[str]
            Email or username of user modifying approval status.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetApprovedBillResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.bill.set_approved_bill(
                approved="true",
                id_bill=285,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.set_approved_bill(
            approved, id_bill, email=email, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.pay_method_cloud import PayMethodCloud
from ...types.pay_method_credit import PayMethodCredit
from ...types.pay_method_stored_method import PayMethodStoredMethod

RequestPaymentAuthorizePaymentMethod = typing.Union[PayMethodCredit, PayMethodStoredMethod, PayMethodCloud]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.external_paypoint_id import ExternalPaypointId
from .v_card_get_response_associated_vendor import VCardGetResponseAssociatedVendor


class VCardGetResponse(UniversalBaseModel):
    vcard_sent: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="vcardSent")] = pydantic.Field(
        default=None
    )
    """
    Indicates if the virtual card was sent.
    """

    card_token: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="cardToken")] = pydantic.Field(
        default=None
    )
    """
    A unique token identifier for the card.
    """

    card_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="cardNumber")] = pydantic.Field(
        default=None
    )
    """
    The masked number of the card.
    """

    cvc: typing.Optional[str] = pydantic.Field(default=None)
    """
    Masked Card Verification Code.
    """

    expiration_date: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="expirationDate")] = (
        pydantic.Field(default=None)
    )
    """
    The expiration date of the card.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The current status of the card.
    """

    amount: typing.Optional[float] = pydantic.Field(default=None)
    """
    The initial amount loaded on the card.
    """

    current_balance: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="currentBalance")] = (
        pydantic.Field(default=None)
    )
    """
    The current balance available on the card.
    """

    expense_limit: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="expenseLimit")] = (
        pydantic.Field(default=None)
    )
    """
    The set limit for expenses.
    """

    expense_limit_period: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="expenseLimitPeriod")
    ] = pydantic.Field(default=None)
    """
    The period for the expense limit.
    """

    max_number_of_uses: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="maxNumberOfUses")] = (
        pydantic.Field(default=None)
    )
    """
    Maximum number of uses allowed for the card.
    """

    current_number_of_uses: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="currentNumberOfUses")
    ] = pydantic.Field(default=None)
    """
    The current number of times the card has been used.
    """

    exact_amount: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="exactAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Indicates if only the exact amount is allowed for transactions.
    """

    mcc: typing.Optional[str] = pydantic.Field(default=None)
    """
    Merchant Category Code, if applicable.
    """

    tcc: typing.Optional[str] = pydantic.Field(default=None)
    """
    Transaction Category Code, if applicable.
    """

    misc_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="misc1")] = pydantic.Field(
        default=None
    )
    """
    A miscellaneous field for additional information.
    """

    misc_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="misc2")] = pydantic.Field(
        default=None
    )
    """
    Another miscellaneous field for extra information.
    """

    date_created: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="dateCreated")] = (
        pydantic.Field(default=None)
    )
    """
    The creation date of the record.
    """

    date_modified: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="dateModified")] = (
        pydantic.Field(default=None)
    )
    """
    The last modified date of the record.
    """

    associated_vendor: typing_extensions.Annotated[
        typing.Optional[VCardGetResponseAssociatedVendor], FieldMetadata(alias="associatedVendor")
    ] = pydantic.Field(default=None)
    """
    Information about the associated vendor.
    """

    associated_customer: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="associatedCustomer")
    ] = pydantic.Field(default=None)
    """
    Information about the associated customer, if applicable.
    """

    parent_org_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ParentOrgName")] = (
        pydantic.Field(default=None)
    )
    """
    Name of the parent organization.
    """

    paypoint_dbaname: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaypointDbaname")] = (
        pydantic.Field(default=None)
    )
    """
    The 'Doing Business As' name of the Paypoint.
    """

    paypoint_legalname: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaypointLegalname")] = (
        pydantic.Field(default=None)
    )
    """
    The legal name of the Paypoint.
    """

    paypoint_entryname: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaypointEntryname")] = (
        pydantic.Field(default=None)
    )
    """
    Entry name for the Paypoint, if applicable.
    """

    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointID")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class VCardGetResponseAssociatedVendorSummary(UniversalBaseModel):
    """
    Summary of vendor's billing and transaction status.
    """

    active_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="ActiveBills")] = (
        pydantic.Field(default=None)
    )
    """
    Number of active bills.
    """

    pending_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="PendingBills")] = (
        pydantic.Field(default=None)
    )
    """
    Number of bills pending approval or payment.
    """

    in_transit_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="InTransitBills")] = (
        pydantic.Field(default=None)
    )
    """
    Number of bills in transit.
    """

    paid_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="PaidBills")] = pydantic.Field(
        default=None
    )
    """
    Number of bills that have been paid.
    """

    overdue_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="OverdueBills")] = (
        pydantic.Field(default=None)
    )
    """
    Number of bills that are overdue.
    """

    approved_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="ApprovedBills")] = (
        pydantic.Field(default=None)
    )
    """
    Number of bills that have been approved.
    """

    disapproved_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="DisapprovedBills")] = (
        pydantic.Field(default=None)
    )
    """
    Number of bills that have been disapproved.
    """

    total_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="TotalBills")] = pydantic.Field(
        default=None
    )
    """
    Total number of bills.
    """

    active_bills_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="ActiveBillsAmount")
    ] = pydantic.Field(default=None)
    """
    Total amount of active bills.
    """

    pending_bills_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="PendingBillsAmount")
    ] = pydantic.Field(default=None)
    """
    Total amount of pending bills.
    """

    in_transit_bills_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="InTransitBillsAmount")
    ] = pydantic.Field(default=None)
    """
    Total amount of bills in transit.
    """

    paid_bills_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="PaidBillsAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Total amount of paid bills.
    """

    overdue_bills_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="OverdueBillsAmount")
    ] = pydantic.Field(default=None)
    """
    Total amount of overdue bills.
    """

    approved_bills_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="ApprovedBillsAmount")
    ] = pydantic.Field(default=None)
    """
    Total amount of approved bills.
    """

    disapproved_bills_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="DisapprovedBillsAmount")
    ] = pydantic.Field(default=None)
    """
    Total amount of rejected bills.
    """

    total_bills_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="TotalBillsAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Total amount of all bills.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data import AdditionalData
from .email import Email
from .language import Language
from .mfa_data import MfaData
from .name_user import NameUser
from .org_scope import OrgScope
from .phone_number import PhoneNumber
from .timezone import Timezone
from .usr_access import UsrAccess
from .usr_status import UsrStatus


class UserData(UniversalBaseModel):
    access: typing.Optional[typing.List[UsrAccess]] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[AdditionalData], FieldMetadata(alias="additionalData")
    ] = None
    email: typing.Optional[Email] = pydantic.Field(default=None)
    """
    The user's email address.
    """

    language: typing.Optional[Language] = None
    mfa_data: typing_extensions.Annotated[typing.Optional[MfaData], FieldMetadata(alias="mfaData")] = None
    name: typing.Optional[NameUser] = None
    phone: typing.Optional[PhoneNumber] = pydantic.Field(default=None)
    """
    The user's phone number.
    """

    pwd: typing.Optional[str] = None
    scope: typing.Optional[typing.List[OrgScope]] = None
    time_zone: typing_extensions.Annotated[typing.Optional[Timezone], FieldMetadata(alias="timeZone")] = None
    usr_status: typing_extensions.Annotated[typing.Optional[UsrStatus], FieldMetadata(alias="usrStatus")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .read_only import ReadOnly
from .required_element import RequiredElement
from .value_templates import ValueTemplates


class LinkData(UniversalBaseModel):
    ro: typing.Optional[ReadOnly] = None
    rq: typing.Optional[RequiredElement] = None
    validator: typing.Optional[str] = pydantic.Field(default=None)
    """
    The type of validation applied to the field. Available values: 
    - text
    - alpha
    - ein
    - url
    - phone
    - alphanumeric
    - zipcode
    - numbers
    - float
    - ssn
    - email
    - routing
    """

    value: typing.Optional[ValueTemplates] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.boarding_application_attachments import BoardingApplicationAttachments
from ...types.created_at import CreatedAt
from ...types.email import Email


class ChargeBackResponse(UniversalBaseModel):
    attachments: typing.Optional[BoardingApplicationAttachments] = pydantic.Field(default=None)
    """
    Object with attached files to response
    """

    contact_email: typing_extensions.Annotated[typing.Optional[Email], FieldMetadata(alias="contactEmail")] = (
        pydantic.Field(default=None)
    )
    """
    Email of response submitter.
    """

    contact_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="contactName")] = (
        pydantic.Field(default=None)
    )
    """
    Name of response submitter
    """

    created_at: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="createdAt")] = (
        pydantic.Field(default=None)
    )
    """
    Timestamp when response was submitted, in UTC.
    """

    id: typing.Optional[int] = pydantic.Field(default=None)
    """
    Chargeback response identifier
    """

    notes: typing.Optional[str] = pydantic.Field(default=None)
    """
    Response notes
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .bill_data import BillData
from .created_at import CreatedAt
from .datetime_nullable import DatetimeNullable
from .dbaname import Dbaname
from .entrypage_id import EntrypageId
from .entrypointfield import Entrypointfield
from .external_paypoint_id import ExternalPaypointId
from .general_events import GeneralEvents
from .last_modified import LastModified
from .legalname import Legalname
from .netamountnullable import Netamountnullable
from .org_parent_name import OrgParentName
from .paypoint_id import PaypointId
from .query_payment_data import QueryPaymentData
from .query_transaction_payor_data import QueryTransactionPayorData
from .source import Source


class SubscriptionQueryRecords(UniversalBaseModel):
    created_at: typing_extensions.Annotated[typing.Optional[CreatedAt], FieldMetadata(alias="CreatedAt")] = (
        pydantic.Field(default=None)
    )
    """
    Timestamp of when the subscription ws created, in UTC.
    """

    customer: typing_extensions.Annotated[
        typing.Optional[QueryTransactionPayorData], FieldMetadata(alias="Customer")
    ] = None
    end_date: typing_extensions.Annotated[typing.Optional[DatetimeNullable], FieldMetadata(alias="EndDate")] = (
        pydantic.Field(default=None)
    )
    """
    The subscription's end date.
    """

    entrypage_id: typing_extensions.Annotated[typing.Optional[EntrypageId], FieldMetadata(alias="EntrypageId")] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="ExternalPaypointID")
    ] = None
    fee_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="FeeAmount")] = pydantic.Field(
        default=None
    )
    """
    Fee applied to the subscription.
    """

    frequency: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Frequency")] = pydantic.Field(
        default=None
    )
    """
    The subscription's frequency.
    """

    id_sub: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="IdSub")] = pydantic.Field(
        default=None
    )
    """
    The subscription's ID.
    """

    invoice_data: typing_extensions.Annotated[typing.Optional[BillData], FieldMetadata(alias="InvoiceData")] = None
    last_run: typing_extensions.Annotated[typing.Optional[DatetimeNullable], FieldMetadata(alias="LastRun")] = (
        pydantic.Field(default=None)
    )
    """
    The last time the subscription was processed.
    """

    last_updated: typing_extensions.Annotated[typing.Optional[LastModified], FieldMetadata(alias="LastUpdated")] = (
        pydantic.Field(default=None)
    )
    """
    The last date and time the subscription was updated.
    """

    left_cycles: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="LeftCycles")] = pydantic.Field(
        default=None
    )
    """
    The number of cycles the subscription has left.
    """

    method: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="Method")] = pydantic.Field(
        default=None
    )
    """
    The subscription's payment method.
    """

    net_amount: typing_extensions.Annotated[typing.Optional[Netamountnullable], FieldMetadata(alias="NetAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The subscription amount, minus any fees.
    """

    next_date: typing_extensions.Annotated[typing.Optional[DatetimeNullable], FieldMetadata(alias="NextDate")] = (
        pydantic.Field(default=None)
    )
    """
    The next date the subscription will be processed.
    """

    parent_org_name: typing_extensions.Annotated[
        typing.Optional[OrgParentName], FieldMetadata(alias="ParentOrgName")
    ] = None
    payment_data: typing_extensions.Annotated[typing.Optional[QueryPaymentData], FieldMetadata(alias="PaymentData")] = (
        None
    )
    paypoint_dbaname: typing_extensions.Annotated[typing.Optional[Dbaname], FieldMetadata(alias="PaypointDbaname")] = (
        pydantic.Field(default=None)
    )
    """
    The paypoint's DBA name.
    """

    paypoint_entryname: typing_extensions.Annotated[
        typing.Optional[Entrypointfield], FieldMetadata(alias="PaypointEntryname")
    ] = pydantic.Field(default=None)
    """
    The paypoint's entryname.
    """

    paypoint_id: typing_extensions.Annotated[typing.Optional[PaypointId], FieldMetadata(alias="PaypointId")] = None
    paypoint_legalname: typing_extensions.Annotated[
        typing.Optional[Legalname], FieldMetadata(alias="PaypointLegalname")
    ] = pydantic.Field(default=None)
    """
    The paypoint's legal name.
    """

    plan_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="PlanId")] = pydantic.Field(
        default=None
    )
    """
    Payment plan ID.
    """

    source: typing_extensions.Annotated[typing.Optional[Source], FieldMetadata(alias="Source")] = None
    start_date: typing_extensions.Annotated[typing.Optional[DatetimeNullable], FieldMetadata(alias="StartDate")] = (
        pydantic.Field(default=None)
    )
    """
    The subscription start date.
    """

    sub_events: typing_extensions.Annotated[
        typing.Optional[typing.List[GeneralEvents]], FieldMetadata(alias="SubEvents")
    ] = pydantic.Field(default=None)
    """
    Events associated with the subscription.
    """

    sub_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="SubStatus")] = pydantic.Field(
        default=None
    )
    """
    The subscription's status.
    - 0: Paused
    - 1: Active
    """

    total_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="TotalAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The subscription amount, including any fees.
    """

    total_cycles: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="TotalCycles")] = (
        pydantic.Field(default=None)
    )
    """
    The total number of cycles the subscription is set to run.
    """

    until_cancelled: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="UntilCancelled")] = (
        pydantic.Field(default=None)
    )
    """
    When `true`, the subscription has no explicit end date and will run until canceled.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .additional_data_map import AdditionalDataMap
from .contacts_response import ContactsResponse
from .email import Email
from .internal_reference_id import InternalReferenceId
from .location_code import LocationCode
from .mcc import Mcc
from .payee_name import PayeeName
from .remitaddress_1 import Remitaddress1
from .remitaddress_2 import Remitaddress2
from .remitcity import Remitcity
from .remitcountry import Remitcountry
from .remitstate import Remitstate
from .remitzip import Remitzip
from .vendor_data_response_payment_method import VendorDataResponsePaymentMethod
from .vendor_number import VendorNumber
from .vendor_response_billing_data import VendorResponseBillingData
from .vendor_response_stored_method import VendorResponseStoredMethod
from .vendor_response_summary import VendorResponseSummary
from .vendorid import Vendorid
from .vendorstatus import Vendorstatus


class VendorDataResponse(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from payabli import (
        ContactsResponse,
        VendorDataResponse,
        VendorResponseBillingData,
        VendorResponseSummary,
    )

    VendorDataResponse(
        vendor_number="1234",
        name_1="Herman's Coatings and Masonry",
        name_2="",
        ein="XXXX6789",
        phone="5555555555",
        email="contact@hermanscoatings.com",
        address_1="123 Ocean Drive",
        address_2="Suite 400",
        city="Miami",
        state="FL",
        zip="33139",
        country="US",
        mcc="7777",
        location_code="MIA123",
        contacts=[
            ContactsResponse(
                contact_name="Herman Martinez",
                contact_email="herman@hermanscoatings.com",
                contact_title="Owner",
                contact_phone="3055550000",
            )
        ],
        billing_data=VendorResponseBillingData(
            id=123,
            nickname="Checking Account",
            bank_name="Country Bank",
            routing_account="123123123",
            account_number="1XXXXXX3123",
            type_account="Checking",
            bank_account_holder_name="Gruzya Adventure Outfitters LLC",
            bank_account_holder_type="Business",
            bank_account_function=0,
            verified=True,
            status=1,
            services=[],
            default=True,
        ),
        payment_method="vcard",
        vendor_status=1,
        vendor_id=1234,
        summary=VendorResponseSummary(
            active_bills=5,
            pending_bills=2,
            in_transit_bills=1,
            paid_bills=10,
            overdue_bills=0,
            approved_bills=3,
            disapproved_bills=0,
            total_bills=21,
            active_bills_amount=1500.0,
            pending_bills_amount=500.0,
            in_transit_bills_amount=200.0,
            paid_bills_amount=3000.0,
            overdue_bills_amount=0.0,
            approved_bills_amount=800.0,
            disapproved_bills_amount=0.0,
            total_bills_amount=6000.0,
        ),
        paypoint_legalname="Gruzya Adventure Outfitters LLC",
        paypoint_dbaname="Gruzya Adventure Outfitters",
        paypoint_entryname="41035afaa7",
        parent_org_name="Pilgrim Planner",
        parent_org_id=1232,
        created_date=datetime.datetime.fromisoformat(
            "2022-07-01 15:00:01+00:00",
        ),
        last_updated=datetime.datetime.fromisoformat(
            "2022-07-01 15:00:01+00:00",
        ),
        remit_address_1="123 Walnut Street",
        remit_address_2="Suite 900",
        remit_city="Miami",
        remit_state="FL",
        remit_zip="31113",
        remit_country="US",
        payee_name_1="Herman Martinez",
        payee_name_2="",
        custom_field_1="",
        custom_field_2="",
        customer_vendor_account="A-37622",
        internal_reference_id=123,
        additional_data={
            "customField": "Custom Value 1",
            "reference": "REF-12345",
            "notes": "Additional vendor information",
        },
        external_paypoint_id="ext123",
        stored_methods=[],
    )
    """

    vendor_number: typing_extensions.Annotated[VendorNumber, FieldMetadata(alias="VendorNumber")]
    name_1: typing_extensions.Annotated[str, FieldMetadata(alias="Name1")] = pydantic.Field()
    """
    Primary name for vendor.
    """

    name_2: typing_extensions.Annotated[str, FieldMetadata(alias="Name2")] = pydantic.Field()
    """
    Secondary name for vendor.
    """

    ein: typing_extensions.Annotated[str, FieldMetadata(alias="EIN")] = pydantic.Field()
    """
    EIN/Tax ID for vendor. In responses, this field is masked, and looks like: `"ein": "XXXXX6789"`.
    """

    phone: typing_extensions.Annotated[str, FieldMetadata(alias="Phone")] = pydantic.Field()
    """
    Vendor's phone number.
    """

    email: typing_extensions.Annotated[Email, FieldMetadata(alias="Email")]
    remit_email: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="RemitEmail")] = pydantic.Field(
        default=None
    )
    """
    Email address for remittance
    """

    address_1: typing_extensions.Annotated[str, FieldMetadata(alias="Address1")] = pydantic.Field()
    """
    Vendor's address.
    """

    address_2: typing_extensions.Annotated[str, FieldMetadata(alias="Address2")] = pydantic.Field()
    """
    Additional line for vendor's address.
    """

    city: typing_extensions.Annotated[str, FieldMetadata(alias="City")] = pydantic.Field()
    """
    Vendor's city.
    """

    state: typing_extensions.Annotated[str, FieldMetadata(alias="State")] = pydantic.Field()
    """
    Vendor's state. Must be a two-character state code.
    """

    zip: typing_extensions.Annotated[str, FieldMetadata(alias="Zip")] = pydantic.Field()
    """
    Vendor's zip code.
    """

    country: typing_extensions.Annotated[str, FieldMetadata(alias="Country")] = pydantic.Field()
    """
    Vendor's country. Payabli supports only US and Canadian vendors.
    """

    mcc: typing_extensions.Annotated[Mcc, FieldMetadata(alias="Mcc")]
    location_code: typing_extensions.Annotated[LocationCode, FieldMetadata(alias="LocationCode")]
    contacts: typing_extensions.Annotated[typing.List[ContactsResponse], FieldMetadata(alias="Contacts")] = (
        pydantic.Field()
    )
    """
    Array of objects describing the vendor's contacts.
    """

    billing_data: typing_extensions.Annotated[VendorResponseBillingData, FieldMetadata(alias="BillingData")] = (
        pydantic.Field()
    )
    """
    Object containing vendor's bank information.
    """

    payment_method: typing_extensions.Annotated[
        VendorDataResponsePaymentMethod, FieldMetadata(alias="PaymentMethod")
    ] = pydantic.Field()
    """
    Preferred payment method for vendor.
    """

    vendor_status: typing_extensions.Annotated[Vendorstatus, FieldMetadata(alias="VendorStatus")]
    vendor_id: typing_extensions.Annotated[Vendorid, FieldMetadata(alias="VendorId")]
    enrollment_status: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="EnrollmentStatus")] = (
        pydantic.Field(default=None)
    )
    """
    Vendor enrollment status
    """

    summary: typing_extensions.Annotated[VendorResponseSummary, FieldMetadata(alias="Summary")] = pydantic.Field()
    """
    Vendor bill summary statistics
    """

    paypoint_legalname: typing_extensions.Annotated[str, FieldMetadata(alias="PaypointLegalname")] = pydantic.Field()
    """
    Legal name of the paypoint
    """

    paypoint_dbaname: typing_extensions.Annotated[str, FieldMetadata(alias="PaypointDbaname")] = pydantic.Field()
    """
    DBA name of the paypoint
    """

    paypoint_entryname: typing_extensions.Annotated[str, FieldMetadata(alias="PaypointEntryname")] = pydantic.Field()
    """
    Entry name of the paypoint
    """

    parent_org_name: typing_extensions.Annotated[str, FieldMetadata(alias="ParentOrgName")] = pydantic.Field()
    """
    Name of the parent organization
    """

    parent_org_id: typing_extensions.Annotated[int, FieldMetadata(alias="ParentOrgId")] = pydantic.Field()
    """
    ID of the parent organization
    """

    created_date: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="CreatedDate")] = pydantic.Field()
    """
    Date when vendor was created
    """

    last_updated: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="LastUpdated")] = pydantic.Field()
    """
    Date when vendor was last updated
    """

    remit_address_1: typing_extensions.Annotated[Remitaddress1, FieldMetadata(alias="remitAddress1")]
    remit_address_2: typing_extensions.Annotated[Remitaddress2, FieldMetadata(alias="remitAddress2")]
    remit_city: typing_extensions.Annotated[Remitcity, FieldMetadata(alias="remitCity")]
    remit_state: typing_extensions.Annotated[Remitstate, FieldMetadata(alias="remitState")]
    remit_zip: typing_extensions.Annotated[Remitzip, FieldMetadata(alias="remitZip")]
    remit_country: typing_extensions.Annotated[Remitcountry, FieldMetadata(alias="remitCountry")]
    payee_name_1: typing_extensions.Annotated[PayeeName, FieldMetadata(alias="payeeName1")]
    payee_name_2: typing_extensions.Annotated[PayeeName, FieldMetadata(alias="payeeName2")]
    custom_field_1: typing_extensions.Annotated[str, FieldMetadata(alias="customField1")] = pydantic.Field()
    """
    Custom field 1 for vendor
    """

    custom_field_2: typing_extensions.Annotated[str, FieldMetadata(alias="customField2")] = pydantic.Field()
    """
    Custom field 2 for vendor
    """

    customer_vendor_account: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="customerVendorAccount")
    ] = pydantic.Field(default=None)
    """
    Account number of paypoint in the Vendor side.
    """

    internal_reference_id: typing_extensions.Annotated[InternalReferenceId, FieldMetadata(alias="InternalReferenceId")]
    additional_data: typing_extensions.Annotated[AdditionalDataMap, FieldMetadata(alias="additionalData")]
    external_paypoint_id: typing_extensions.Annotated[str, FieldMetadata(alias="externalPaypointID")] = pydantic.Field()
    """
    External paypoint identifier
    """

    stored_methods: typing_extensions.Annotated[
        typing.List[VendorResponseStoredMethod], FieldMetadata(alias="StoredMethods")
    ] = pydantic.Field()
    """
    Array of stored payment methods for vendor
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
